@@ -4,9 +4,10 @@ from .abstracts import (
     AbstractPaymentWrapper,
     AioTestCase
 )
-from .aiohttp_custom_api import CustomParser
-from .basic_requests_api import HttpXParser, Storage
+from .aiohttp_custom_api import RequestManager
+from .basic_requests_api import HttpXParser
 from .mixins import BillMixin, ToolsMixin
+from .storage import Storage
 
 __all__ = (
     'HttpXParser',
@@ -17,5 +18,5 @@ __all__ = (
     'AioTestCase',
     'BillMixin',
     'ToolsMixin',
-    'CustomParser'
+    'RequestManager'
 )
