@@ -1,11 +1,13 @@
 import abc
 import asyncio
 import unittest
-from typing import AsyncGenerator, Optional, Dict, Any, Union, List
+from typing import AsyncGenerator, Optional, Dict, Any, Union, List, Tuple
 
-from aiohttp import ClientSession
+from aiohttp import ClientSession, web
 from aiohttp.typedefs import LooseCookies
 
+from glQiwiApi import types
+from glQiwiApi.core.web_hooks.handler import HandlerManager
 from glQiwiApi.types import ProxyService, Response
 
 
@@ -14,7 +16,6 @@ class BaseStorage(abc.ABC):
     Абстрактный класс контроллера кэша
 
     """
-    __slots__ = ('tmp_data', '_cache_time')
 
     @abc.abstractmethod
     def get_current(self, key: str) -> Any:
@@ -24,13 +25,7 @@ class BaseStorage(abc.ABC):
     def clear(self, key: str, force: bool = False) -> Any:
         raise NotImplementedError
 
-    @abc.abstractmethod
-    def update_data(
-            self,
-            result: Any,
-            kwargs: Any,
-            status_code: Optional[Union[str, int]] = None
-    ) -> None:
+    def update_data(self, obj_to_cache: Any, key: Any) -> None:
         raise NotImplementedError
 
     def validate(self, kwargs: Dict[str, Any]) -> None:
@@ -144,3 +139,70 @@ class AbstractParser(abc.ABC):
         elif isinstance(self.session, ClientSession):
             if self.session.closed:
                 self.session = ClientSession(**kwargs)
+
+
+class BaseWebHookView(web.View):
+    """
+    Base webhook view for processing events
+    You can make own view and than use it in code,
+    just inheriting this base class
+
+    """
+    app_key_check_ip: Optional[str] = None
+    """app_key_check_ip stores key to a storage"""
+
+    app_key_handler_manager: Optional[str] = None
+    """app_key_handler_manager"""
+
+    def _check_ip(self, ip: str):
+        """_check_ip checks if given IP is in set of allowed ones"""
+        raise NotImplementedError
+
+    def parse_update(self):
+        """parse_update method that deals with marshaling json"""
+        raise NotImplementedError
+
+    def validate_ip(self):
+        """ validating request ip address """
+        if self.request.app.get(self.app_key_check_ip):
+            ip_address, accept = self.check_ip()
+            if not accept:
+                raise web.HTTPUnauthorized()
+
+    def check_ip(self) -> Union[Tuple[str, bool], Tuple[None, bool]]:
+        """ checking ip, using headers or peer_name """
+        forwarded_for = self.request.headers.get("X-Forwarded-For")
+        if forwarded_for:
+            return forwarded_for, self._check_ip(forwarded_for)
+        peer_name = self.request.transport.get_extra_info("peername")
+
+        if peer_name is not None:
+            host, _ = peer_name
+            return host, self._check_ip(host)
+
+        return None, False
+
+    @abc.abstractmethod
+    async def post(self) -> None:
+        """
+        Process POST request with basic IP validation.
+
+        """
+        self.validate_ip()
+
+        update = await self.parse_update()
+
+        self._hash_validator(update)
+
+        await self.handler_manager.process_event(update)
+
+    def _hash_validator(self, update: Union[
+        types.Notification, types.WebHook
+    ]) -> None:
+        """ Validating by hash of update """
+
+    @property
+    def handler_manager(self) -> "HandlerManager":
+        """ Return handler manager """
+        return self.request.app.get(
+            self.app_key_handler_manager)  # type: ignore
