@@ -28,7 +28,10 @@ from .qiwi_types import (
     RefundBill,
     Polygon,
     Terminal,
-    Partner
+    Partner,
+    WebHookConfig,
+    WebHook,
+    Notification
 )
 from .yoomoney_types import (
     OperationType,
@@ -89,6 +92,9 @@ __all__ = [
     'Polygon',
     'Terminal',
     'Partner',
+    'WebHookConfig',
+    'WebHook',
+    'Notification',
     'Executors',
     'FuncT'
 ]
