@@ -11,8 +11,15 @@ class Statistic(BaseModel):
     out: List[Sum] = Field(alias="outgoingTotal")
 
     class Config:
+        """ Pydantic config """
         json_loads = custom_load
 
+        def __str__(self) -> str:
+            return f'Config class with loads={self.json_loads}'
+
+        def __repr__(self) -> str:
+            return self.__str__()
+
 
 __all__ = [
     'Statistic'
