@@ -7,9 +7,11 @@ DEFAULT_YOOMONEY_HEADERS = {
 }
 
 ERROR_CODE_NUMBERS = {
-    '400': 'Ошибка, связанная с типом запроса к апи, возможно вы передали недействительный API токен.',
-    '401': 'Указан несуществующий, просроченный, или отозванный токен.',
-    '403': 'Запрошена операция, на которую у токена нет прав.'
+    "400": "Ошибка, связанная с типом запроса к апи,"
+           "возможно вы передали недействительный API токен.",
+    "401": "Указан несуществующий, просроченный, или отозванный токен.",
+    "403": "Запрошена операция, на которую у токена нет прав.",
+    "400_special_bad_proxy": "Ошибка, связанная с использованием прокси"
 }
 
 OPERATION_TRANSFER = {
